@@ -16,7 +16,6 @@
  */
 package org.apache.catalina.loader;
 
-import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
 import java.io.File;
@@ -28,7 +27,8 @@ import java.net.URLClassLoader;
 import java.nio.charset.StandardCharsets;
 
 import javax.management.ObjectName;
-import javax.servlet.ServletContext;
+
+import jakarta.servlet.ServletContext;
 
 import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
@@ -39,6 +39,9 @@ import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.catalina.util.ToStringUtil;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.jakartaee.ClassConverter;
+import org.apache.tomcat.jakartaee.EESpecProfile;
+import org.apache.tomcat.jakartaee.EESpecProfiles;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.UDecoder;
 import org.apache.tomcat.util.compat.JreCompat;
@@ -46,47 +49,25 @@ import org.apache.tomcat.util.modeler.Registry;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * Classloader implementation which is specialized for handling web applications in the most efficient way, while being
- * Catalina aware (all accesses to resources are made through {@link org.apache.catalina.WebResourceRoot}). This class
- * loader supports detection of modified Java classes, which can be used to implement auto-reload support.
+ * Classloader implementation which is specialized for handling web
+ * applications in the most efficient way, while being Catalina aware (all
+ * accesses to resources are made through
+ * {@link org.apache.catalina.WebResourceRoot}).
+ * This class loader supports detection of modified
+ * Java classes, which can be used to implement auto-reload support.
  * <p>
- * This class loader is configured via the Resources children of its Context prior to calling <code>start()</code>. When
- * a new class is required, these Resources will be consulted first to locate the class. If it is not present, the
- * system class loader will be used instead.
+ * This class loader is configured via the Resources children of its Context
+ * prior to calling <code>start()</code>.  When a new class is required,
+ * these Resources will be consulted first to locate the class.  If it
+ * is not present, the system class loader will be used instead.
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
  */
-public class WebappLoader extends LifecycleMBeanBase implements Loader, PropertyChangeListener {
+public class WebappLoader extends LifecycleMBeanBase implements Loader{
 
     private static final Log log = LogFactory.getLog(WebappLoader.class);
 
-    // ----------------------------------------------------------- Constructors
-
-    /**
-     * Construct a new WebappLoader. The parent class loader will be defined by {@link Context#getParentClassLoader()}.
-     */
-    public WebappLoader() {
-        this(null);
-    }
-
-
-    /**
-     * Construct a new WebappLoader with the specified class loader to be defined as the parent of the ClassLoader we
-     * ultimately create.
-     *
-     * @param parent The parent class loader
-     *
-     * @deprecated Use {@link Context#setParentClassLoader(ClassLoader)} to specify the required class loader. This
-     *                 method will be removed in Tomcat 10 onwards.
-     */
-    @Deprecated
-    public WebappLoader(ClassLoader parent) {
-        super();
-        this.parentClassLoader = parent;
-    }
-
-
     // ----------------------------------------------------- Instance Variables
 
     /**
@@ -102,30 +83,28 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
 
     /**
-     * The "follow standard delegation model" flag that will be used to configure our ClassLoader.
+     * The "follow standard delegation model" flag that will be used to
+     * configure our ClassLoader.
      */
     private boolean delegate = false;
 
 
     /**
-     * The Java class name of the ClassLoader implementation to be used. This class should extend WebappClassLoaderBase,
-     * otherwise, a different loader implementation must be used.
+     * The profile name which will be used by the converter, or null if not used.
+     * Any invalid profile value will default to the TOMCAT profile, which
+     * converts all packages used by Tomcat.
+     */
+    private String jakartaConverter = null;
+
+
+    /**
+     * The Java class name of the ClassLoader implementation to be used.
+     * This class should extend WebappClassLoaderBase, otherwise, a different
+     * loader implementation must be used.
      */
     private String loaderClass = ParallelWebappClassLoader.class.getName();
 
 
-    /**
-     * The parent class loader of the class loader we will create.
-     */
-    private ClassLoader parentClassLoader = null;
-
-
-    /**
-     * The reloadable flag for this Loader.
-     */
-    private boolean reloadable = false;
-
-
     /**
      * The string manager for this package.
      */
@@ -146,6 +125,9 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
     // ------------------------------------------------------------- Properties
 
+    /**
+     * Return the Java class loader to be used by this Container.
+     */
     @Override
     public ClassLoader getClassLoader() {
         return classLoader;
@@ -169,35 +151,61 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
             throw new IllegalStateException(sm.getString("webappLoader.setContext.ise"));
         }
 
-        // Deregister from the old Context (if any)
-        if (this.context != null) {
-            this.context.removePropertyChangeListener(this);
-        }
-
         // Process this property change
         Context oldContext = this.context;
         this.context = context;
         support.firePropertyChange("context", oldContext, this.context);
-
-        // Register with the new Container (if any)
-        if (this.context != null) {
-            setReloadable(this.context.getReloadable());
-            this.context.addPropertyChangeListener(this);
-        }
     }
 
 
+    /**
+     * Return the "follow standard delegation model" flag used to configure
+     * our ClassLoader.
+     */
     @Override
     public boolean getDelegate() {
         return this.delegate;
     }
 
 
+    /**
+     * Set the "follow standard delegation model" flag used to configure
+     * our ClassLoader.
+     *
+     * @param delegate The new flag
+     */
     @Override
     public void setDelegate(boolean delegate) {
         boolean oldDelegate = this.delegate;
         this.delegate = delegate;
-        support.firePropertyChange("delegate", Boolean.valueOf(oldDelegate), Boolean.valueOf(this.delegate));
+        support.firePropertyChange("delegate", Boolean.valueOf(oldDelegate),
+                                   Boolean.valueOf(this.delegate));
+    }
+
+
+    /**
+     * @return a non null String if the loader will attempt to use the
+     *  Jakarta converter. The String is the name of the profile
+     *  used for conversion.
+     */
+    public String getJakartaConverter() {
+        return jakartaConverter;
+    }
+
+
+    /**
+     * Set the Jakarta converter.
+     *
+     * @param jakartaConverter The profile name which will be used by the converter
+     *   Any invalid profile value will default to the TOMCAT profile, which
+     *   converts all packages used by Tomcat.
+     */
+    public void setJakartaConverter(String jakartaConverter) {
+        String oldJakartaConverter = this.jakartaConverter;
+        this.jakartaConverter = jakartaConverter;
+        support.firePropertyChange("jakartaConverter",
+                oldJakartaConverter,
+                this.jakartaConverter);
     }
 
 
@@ -219,8 +227,9 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     }
 
     /**
-     * Set the ClassLoader instance, without relying on reflection This method will also invoke
-     * {@link #setLoaderClass(String)} with {@code loaderInstance.getClass().getName()} as an argument
+     * Set the ClassLoader instance, without relying on reflection
+     * This method will also invoke {@link #setLoaderClass(String)} with
+     * {@code loaderInstance.getClass().getName()} as an argument
      *
      * @param loaderInstance The new ClassLoader instance to use
      */
@@ -229,32 +238,13 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
         setLoaderClass(loaderInstance.getClass().getName());
     }
 
-
-    /**
-     * Return the reloadable flag for this Loader.
-     */
-    @Override
-    public boolean getReloadable() {
-        return this.reloadable;
-    }
-
-
-    /**
-     * Set the reloadable flag for this Loader.
-     *
-     * @param reloadable The new reloadable flag
-     */
-    @Override
-    public void setReloadable(boolean reloadable) {
-        // Process this property change
-        boolean oldReloadable = this.reloadable;
-        this.reloadable = reloadable;
-        support.firePropertyChange("reloadable", Boolean.valueOf(oldReloadable), Boolean.valueOf(this.reloadable));
-    }
-
-
     // --------------------------------------------------------- Public Methods
 
+    /**
+     * Add a property change listener to this component.
+     *
+     * @param listener The listener to add
+     */
     @Override
     public void addPropertyChangeListener(PropertyChangeListener listener) {
 
@@ -263,18 +253,22 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     }
 
 
+    /**
+     * Execute a periodic task, such as reloading, etc. This method will be
+     * invoked inside the classloading context of this container. Unexpected
+     * throwables will be caught and logged.
+     */
     @Override
     public void backgroundProcess() {
-        if (reloadable && modified()) {
-            Thread currentThread = Thread.currentThread();
-            try {
-                currentThread.setContextClassLoader(WebappLoader.class.getClassLoader());
-                if (context != null) {
+        Context context = getContext();
+        if (context != null) {
+            if (context.getReloadable() && modified()) {
+                ClassLoader originalTccl = Thread.currentThread().getContextClassLoader();
+                try {
+                    Thread.currentThread().setContextClassLoader(WebappLoader.class.getClassLoader());
                     context.reload();
-                }
-            } finally {
-                if (context != null && context.getLoader() != null) {
-                    currentThread.setContextClassLoader(context.getLoader().getClassLoader());
+                } finally {
+                    Thread.currentThread().setContextClassLoader(originalTccl);
                 }
             }
         }
@@ -294,8 +288,8 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     }
 
     public String getLoaderRepositoriesString() {
-        String[] repositories = getLoaderRepositories();
-        StringBuilder sb = new StringBuilder();
+        String repositories[]=getLoaderRepositories();
+        StringBuilder sb=new StringBuilder();
         for (String repository : repositories) {
             sb.append(repository).append(':');
         }
@@ -304,7 +298,8 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
 
     /**
-     * Classpath, as set in org.apache.catalina.jsp_classpath context property
+     * Classpath, as set in org.apache.catalina.jsp_classpath context
+     * property
      *
      * @return The classpath
      */
@@ -313,18 +308,30 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     }
 
 
+    /**
+     * Has the internal repository associated with this Loader been modified,
+     * such that the loaded classes should be reloaded?
+     */
     @Override
     public boolean modified() {
-        return classLoader != null && classLoader.modified();
+        return classLoader != null ? classLoader.modified() : false ;
     }
 
 
+    /**
+     * Remove a property change listener from this component.
+     *
+     * @param listener The listener to remove
+     */
     @Override
     public void removePropertyChangeListener(PropertyChangeListener listener) {
         support.removePropertyChangeListener(listener);
     }
 
 
+    /**
+     * Return a String representation of this component.
+     */
     @Override
     public String toString() {
         return ToStringUtil.toString(this, context);
@@ -332,11 +339,11 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
 
     /**
-     * Start associated {@link ClassLoader} and implement the requirements of
-     * {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
+     * Start associated {@link ClassLoader} and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
-     * @exception LifecycleException if this component detects a fatal error that prevents this component from being
-     *                                   used
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
     @Override
     protected void startInternal() throws LifecycleException {
@@ -358,6 +365,18 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
             classLoader.setResources(context.getResources());
             classLoader.setDelegate(this.delegate);
 
+            // Set Jakarta class converter
+            if (getJakartaConverter() != null) {
+                EESpecProfile profile = null;
+                try {
+                    profile = EESpecProfiles.valueOf(getJakartaConverter());
+                } catch (IllegalArgumentException ignored) {
+                    // Use default value
+                    log.warn(sm.getString("webappLoader.unknownProfile", getJakartaConverter()));
+                }
+                classLoader.addTransformer((profile != null) ? new ClassConverter(profile) : new ClassConverter());
+            }
+
             // Configure our repositories
             setClassPath();
 
@@ -369,15 +388,16 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
             if (!contextName.startsWith("/")) {
                 contextName = "/" + contextName;
             }
-            ObjectName cloname =
-                    new ObjectName(context.getDomain() + ":type=" + classLoader.getClass().getSimpleName() + ",host=" +
-                            context.getParent().getName() + ",context=" + contextName);
-            Registry.getRegistry(null).registerComponent(classLoader, cloname, null);
+            ObjectName cloname = new ObjectName(context.getDomain() + ":type=" +
+                    classLoader.getClass().getSimpleName() + ",host=" +
+                    context.getParent().getName() + ",context=" + contextName);
+            Registry.getRegistry(null, null)
+                .registerComponent(classLoader, cloname, null);
 
         } catch (Throwable t) {
-            Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
-            ExceptionUtils.handleThrowable(throwable);
-            throw new LifecycleException(sm.getString("webappLoader.startError"), throwable);
+            t = ExceptionUtils.unwrapInvocationTargetException(t);
+            ExceptionUtils.handleThrowable(t);
+            throw new LifecycleException(sm.getString("webappLoader.startError"), t);
         }
 
         setState(LifecycleState.STARTING);
@@ -385,11 +405,11 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
 
     /**
-     * Stop associated {@link ClassLoader} and implement the requirements of
-     * {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
+     * Stop associated {@link ClassLoader} and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
-     * @exception LifecycleException if this component detects a fatal error that prevents this component from being
-     *                                   used
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
     @Override
     protected void stopInternal() throws LifecycleException {
@@ -418,10 +438,10 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
                 if (!contextName.startsWith("/")) {
                     contextName = "/" + contextName;
                 }
-                ObjectName cloname =
-                        new ObjectName(context.getDomain() + ":type=" + classLoader.getClass().getSimpleName() +
-                                ",host=" + context.getParent().getName() + ",context=" + contextName);
-                Registry.getRegistry(null).unregisterComponent(cloname);
+                ObjectName cloname = new ObjectName(context.getDomain() + ":type=" +
+                        classLoader.getClass().getSimpleName() + ",host=" +
+                        context.getParent().getName() + ",context=" + contextName);
+                Registry.getRegistry(null, null).unregisterComponent(cloname);
             } catch (Exception e) {
                 log.warn(sm.getString("webappLoader.stopError"), e);
             }
@@ -432,61 +452,33 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     }
 
 
-    // ----------------------------------------- PropertyChangeListener Methods
-
-
-    /**
-     * Process property change events from our associated Context.
-     *
-     * @param event The property change event that has occurred
-     */
-    @Override
-    public void propertyChange(PropertyChangeEvent event) {
-
-        // Validate the source of this event
-        if (!(event.getSource() instanceof Context)) {
-            return;
-        }
-
-        // Process a relevant property change
-        if (event.getPropertyName().equals("reloadable")) {
-            try {
-                setReloadable(((Boolean) event.getNewValue()).booleanValue());
-            } catch (NumberFormatException e) {
-                log.error(sm.getString("webappLoader.reloadable", event.getNewValue().toString()));
-            }
-        }
-    }
-
-
     // ------------------------------------------------------- Private Methods
 
     /**
      * Create associated classLoader.
      */
-    private WebappClassLoaderBase createClassLoader() throws Exception {
+    private WebappClassLoaderBase createClassLoader()
+        throws Exception {
 
         if (classLoader != null) {
             return classLoader;
         }
 
-        if (parentClassLoader == null) {
-            parentClassLoader = context.getParentClassLoader();
-        } else {
-            context.setParentClassLoader(parentClassLoader);
-        }
-
         if (ParallelWebappClassLoader.class.getName().equals(loaderClass)) {
-            return new ParallelWebappClassLoader(parentClassLoader);
+            return new ParallelWebappClassLoader(context.getParentClassLoader());
         }
 
         Class<?> clazz = Class.forName(loaderClass);
+        WebappClassLoaderBase classLoader = null;
+
+        ClassLoader parentClassLoader = context.getParentClassLoader();
 
         Class<?>[] argTypes = { ClassLoader.class };
         Object[] args = { parentClassLoader };
         Constructor<?> constr = clazz.getConstructor(argTypes);
+        classLoader = (WebappClassLoaderBase) constr.newInstance(args);
 
-        return (WebappClassLoaderBase) constr.newInstance(args);
+        return classLoader;
     }
 
 
@@ -506,25 +498,30 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
         ServletContext servletContext = context.getServletContext();
 
         // Assigning permissions for the work directory
-        File workDir = (File) servletContext.getAttribute(ServletContext.TEMPDIR);
+        File workDir =
+            (File) servletContext.getAttribute(ServletContext.TEMPDIR);
         if (workDir != null) {
             try {
                 String workDirPath = workDir.getCanonicalPath();
-                classLoader.addPermission(new FilePermission(workDirPath, "read,write"));
-                classLoader.addPermission(new FilePermission(workDirPath + File.separator + "-", "read,write,delete"));
+                classLoader.addPermission
+                    (new FilePermission(workDirPath, "read,write"));
+                classLoader.addPermission
+                    (new FilePermission(workDirPath + File.separator + "-",
+                                        "read,write,delete"));
             } catch (IOException e) {
                 // Ignore
             }
         }
 
         for (URL url : context.getResources().getBaseUrls()) {
-            classLoader.addPermission(url);
+           classLoader.addPermission(url);
         }
     }
 
 
     /**
-     * Set the appropriate context attribute for our class path. This is required only because Jasper depends on it.
+     * Set the appropriate context attribute for our class path.  This
+     * is required only because Jasper depends on it.
      */
     private void setClassPath() {
 
@@ -571,7 +568,7 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
     private boolean buildClassPath(StringBuilder classpath, ClassLoader loader) {
         if (loader instanceof URLClassLoader) {
-            URL[] repositories = ((URLClassLoader) loader).getURLs();
+            URL repositories[] = ((URLClassLoader) loader).getURLs();
             for (URL url : repositories) {
                 String repository = url.toString();
                 if (repository.startsWith("file://")) {
@@ -589,11 +586,11 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
                 }
                 classpath.append(repository);
             }
-        } else if (loader == ClassLoader.getSystemClassLoader()) {
-            // Java 9 onwards. The internal class loaders no longer extend
+        } else if (loader == ClassLoader.getSystemClassLoader()){
+            // From Java 9 the internal class loaders no longer extend
             // URLCLassLoader
             String cp = System.getProperty("java.class.path");
-            if (cp != null && !cp.isEmpty()) {
+            if (cp != null && cp.length() > 0) {
                 if (classpath.length() > 0) {
                     classpath.append(File.pathSeparator);
                 }
