@@ -20,10 +20,10 @@ import java.io.IOException;
 import java.io.PrintWriter;
 import java.util.Enumeration;
 
-import javax.servlet.ServletException;
-import javax.servlet.http.HttpServlet;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.http.HttpServlet;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletResponse;
 
 import org.junit.Assert;
 import org.junit.Test;
@@ -39,7 +39,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
     private static final String CR = "\r";
     private static final String LF = "\n";
-    private static final String CRLF = CR + LF;
+    private  static final String CRLF = CR + LF;
 
     /**
      * Test case for https://bz.apache.org/bugzilla/show_bug.cgi?id=48839
@@ -76,15 +76,13 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
                 connect();
 
                 String[] request = new String[1];
-                // @formatter:off
                 request[0] =
-                        "GET http://localhost:8080/test HTTP/1.1" + CRLF +
-                        "Host: localhost:8080" + CRLF +
-                        "X-Bug48839: abcd" + CRLF +
-                        "\tefgh" + CRLF +
-                        "Connection: close" + CRLF +
-                        CRLF;
-                // @formatter:on
+                    "GET http://localhost:8080/test HTTP/1.1" + CRLF +
+                    "Host: localhost:8080" + CRLF +
+                    "X-Bug48839: abcd" + CRLF +
+                    "\tefgh" + CRLF +
+                    "Connection: close" + CRLF +
+                    CRLF;
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -118,7 +116,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
          * Only interested in the request headers from a GET request
          */
         @Override
-        protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
+        protected void doGet(HttpServletRequest req, HttpServletResponse resp)
+                throws ServletException, IOException {
             // Just echo the header value back as plain text
             resp.setContentType("text/plain");
 
@@ -168,8 +167,9 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
     @Test
     public void testBug51557SeparatorsInName() throws Exception {
-        char httpSeparators[] = new char[] { '\t', ' ', '\"', '(', ')', ',', '/', ':', ';', '<', '=', '>', '?', '@',
-                '[', '\\', ']', '{', '}' };
+        char httpSeparators[] = new char[] {
+                '\t', ' ', '\"', '(', ')', ',', '/', ':', ';', '<',
+                '=', '>', '?', '@', '[', '\\', ']', '{', '}' };
 
         for (char s : httpSeparators) {
             doTestBug51557CharInName(s);
@@ -222,7 +222,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     @Test
     public void testBug51557Continuation() {
 
-        Bug51557Client client = new Bug51557Client("X-Bug=51557NoColon", "foo" + SimpleHttpClient.CRLF + " bar");
+        Bug51557Client client = new Bug51557Client("X-Bug=51557NoColon",
+                "foo" + SimpleHttpClient.CRLF + " bar");
 
         client.doRequest();
         Assert.assertTrue(client.isResponse200());
@@ -234,7 +235,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     @Test
     public void testBug51557BoundaryStart() {
 
-        Bug51557Client client = new Bug51557Client("=X-Bug51557", "invalid");
+        Bug51557Client client = new Bug51557Client("=X-Bug51557",
+                "invalid");
 
         client.doRequest();
         Assert.assertTrue(client.isResponse200());
@@ -246,7 +248,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     @Test
     public void testBug51557BoundaryEnd() {
 
-        Bug51557Client client = new Bug51557Client("X-Bug51557=", "invalid");
+        Bug51557Client client = new Bug51557Client("X-Bug51557=",
+                "invalid");
 
         client.doRequest();
         Assert.assertTrue(client.isResponse200());
@@ -256,7 +259,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
 
     private void doTestBug51557CharInName(char s) {
-        Bug51557Client client = new Bug51557Client("X-Bug" + s + "51557", "invalid");
+        Bug51557Client client =
+            new Bug51557Client("X-Bug" + s + "51557", "invalid");
 
         client.doRequest();
         Assert.assertTrue(client.isResponse200());
@@ -266,7 +270,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
 
     private void doTestBug51557InvalidCharInValue(char s) {
-        Bug51557Client client = new Bug51557Client("X-Bug51557-Invalid", "invalid" + s + "invalid");
+        Bug51557Client client =
+            new Bug51557Client("X-Bug51557-Invalid", "invalid" + s + "invalid");
 
         client.doRequest();
         Assert.assertTrue("Testing [" + (int) s + "]", client.isResponse200());
@@ -276,7 +281,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
 
     private void doTestBug51557ValidCharInValue(char s) {
-        Bug51557Client client = new Bug51557Client("X-Bug51557-Valid", "valid" + s + "valid");
+        Bug51557Client client =
+            new Bug51557Client("X-Bug51557-Valid", "valid" + s + "valid");
 
         client.doRequest();
         Assert.assertTrue("Testing [" + (int) s + "]", client.isResponse200());
@@ -294,17 +300,18 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
         private final String headerLine;
         private final boolean rejectIllegalHeader;
 
-        Bug51557Client(String headerName) {
+        public Bug51557Client(String headerName) {
             this.headerName = headerName;
             this.headerLine = headerName;
             this.rejectIllegalHeader = false;
         }
 
-        Bug51557Client(String headerName, String headerValue) {
+        public Bug51557Client(String headerName, String headerValue) {
             this(headerName, headerValue, false);
         }
 
-        Bug51557Client(String headerName, String headerValue, boolean rejectIllegalHeader) {
+        public Bug51557Client(String headerName, String headerValue,
+                boolean rejectIllegalHeader) {
             this.headerName = headerName;
             this.headerLine = headerName + ": " + headerValue;
             this.rejectIllegalHeader = rejectIllegalHeader;
@@ -315,12 +322,14 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
             Tomcat tomcat = getTomcatInstance();
 
             Context root = tomcat.addContext("", TEMP_DIR);
-            Tomcat.addServlet(root, "Bug51557", new Bug51557Servlet(headerName));
+            Tomcat.addServlet(root, "Bug51557",
+                    new Bug51557Servlet(headerName));
             root.addServletMappingDecoded("/test", "Bug51557");
 
             try {
                 Connector connector = tomcat.getConnector();
-                Assert.assertTrue(connector.setProperty("rejectIllegalHeader", Boolean.toString(rejectIllegalHeader)));
+                Assert.assertTrue(connector.setProperty(
+                        "rejectIllegalHeader", Boolean.toString(rejectIllegalHeader)));
                 tomcat.start();
                 setPort(connector.getLocalPort());
 
@@ -329,15 +338,13 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
                 connect();
 
                 String[] request = new String[1];
-                // @formatter:off
                 request[0] =
-                        "GET http://localhost:8080/test HTTP/1.1" + CRLF +
-                        "Host: localhost:8080" + CRLF +
-                        headerLine + CRLF +
-                        "X-Bug51557: abcd" + CRLF +
-                        "Connection: close" + CRLF +
-                        CRLF;
-                // @formatter:on
+                    "GET http://localhost:8080/test HTTP/1.1" + CRLF +
+                    "Host: localhost:8080" + CRLF +
+                    headerLine + CRLF +
+                    "X-Bug51557: abcd" + CRLF +
+                    "Connection: close" + CRLF +
+                    CRLF;
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -370,9 +377,10 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
         private String invalidHeaderName;
 
         /**
-         * @param invalidHeaderName The header name should be invalid and therefore ignored by the header parsing code
+         * @param invalidHeaderName The header name should be invalid and
+         *                          therefore ignored by the header parsing code
          */
-        Bug51557Servlet(String invalidHeaderName) {
+        public Bug51557Servlet(String invalidHeaderName) {
             this.invalidHeaderName = invalidHeaderName;
         }
 
@@ -380,7 +388,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
          * Only interested in the request headers from a GET request
          */
         @Override
-        protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
+        protected void doGet(HttpServletRequest req, HttpServletResponse resp)
+                throws ServletException, IOException {
             // Just echo the header value back as plain text
             resp.setContentType("text/plain");
 
@@ -390,18 +399,19 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
             processHeaders("X-Bug51557", req, out);
         }
 
-        private void processHeaders(String header, HttpServletRequest req, PrintWriter out) {
+        private void processHeaders(String header, HttpServletRequest req,
+                PrintWriter out) {
             Enumeration<String> values = req.getHeaders(header);
             while (values.hasMoreElements()) {
-                out.print(values.nextElement());
+                out.println(values.nextElement());
             }
         }
     }
 
 
     /**
-     * Test case for new lines at the start of a request. RFC2616 does not permit any, but Tomcat is tolerant of them if
-     * they are present.
+     * Test case for new lines at the start of a request. RFC2616
+     * does not permit any, but Tomcat is tolerant of them if they are present.
      */
     @Test
     public void testNewLines() {
@@ -415,8 +425,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
 
     /**
-     * Test case for new lines at the start of a request. RFC2616 does not permit any, but Tomcat is tolerant of them if
-     * they are present.
+     * Test case for new lines at the start of a request. RFC2616
+     * does not permit any, but Tomcat is tolerant of them if they are present.
      */
     @Test
     public void testNewLinesExcessive() {
@@ -461,16 +471,14 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
                 connect();
 
                 String[] request = new String[1];
-                // @formatter:off
                 request[0] =
-                        newLines +
-                        "GET http://localhost:8080/test HTTP/1.1" + CRLF +
-                        "Host: localhost:8080" + CRLF +
-                        "X-Bug48839: abcd" + CRLF +
-                        "\tefgh" + CRLF +
-                        "Connection: close" + CRLF +
-                        CRLF;
-                // @formatter:on
+                    newLines +
+                    "GET http://localhost:8080/test HTTP/1.1" + CRLF +
+                    "Host: localhost:8080" + CRLF +
+                    "X-Bug48839: abcd" + CRLF +
+                    "\tefgh" + CRLF +
+                    "Connection: close" + CRLF +
+                    CRLF;
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -533,7 +541,10 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
                 String[] request = new String[2];
                 request[0] = "GET http://localhost:8080/test HTTP/1.1" + CR;
-                request[1] = LF + "Host: localhost:8080" + CRLF + "Connection: close" + CRLF + CRLF;
+                request[1] = LF +
+                        "Host: localhost:8080" + CRLF +
+                        "Connection: close" + CRLF +
+                        CRLF;
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -600,8 +611,10 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
                 connect();
 
                 String[] request = new String[1];
-                request[0] = "GET http://localhost:8080/test HTTP/1.1" + CRLF + "Host: localhost:8080" + CRLF +
-                        "X-Header: Ignore" + CRLF + "X-Header" + (char) 130 + ": Broken" + CRLF + CRLF;
+                request[0] = "GET http://localhost:8080/test HTTP/1.1" + CRLF +
+                        "Host: localhost:8080" + CRLF +
+                        "X-Header: Ignore" + CRLF +
+                        "X-Header" + (char) 130 + ": Broken" + CRLF + CRLF;
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -631,8 +644,11 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     public void testInvalidMethod() {
 
         String[] request = new String[1];
-        request[0] = "GET" + (char) 0 + " /test HTTP/1.1" + CRLF + "Host: localhost:8080" + CRLF + "Connection: close" +
-                CRLF + CRLF;
+        request[0] =
+            "GET" + (char) 0 + " /test HTTP/1.1" + CRLF +
+            "Host: localhost:8080" + CRLF +
+            "Connection: close" + CRLF +
+            CRLF;
 
         InvalidClient client = new InvalidClient(request);
 
@@ -660,7 +676,11 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     public void testInvalidEndOfRequestLine01() {
 
         String[] request = new String[1];
-        request[0] = "GET /test HTTP/1.1" + CR + "Host: localhost:8080" + CRLF + "Connection: close" + CRLF + CRLF;
+        request[0] =
+                "GET /test HTTP/1.1" + CR +
+                "Host: localhost:8080" + CRLF +
+                "Connection: close" + CRLF +
+                CRLF;
 
         InvalidClient client = new InvalidClient(request);
 
@@ -674,8 +694,12 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     public void testInvalidHeader01() {
 
         String[] request = new String[1];
-        request[0] = "GET /test HTTP/1.1" + CRLF + "Host: localhost:8080" + CRLF + CR + "X-Header: xxx" + CRLF +
-                "Connection: close" + CRLF + CRLF;
+        request[0] =
+                "GET /test HTTP/1.1" + CRLF +
+                "Host: localhost:8080" + CRLF +
+                CR + "X-Header: xxx" + CRLF +
+                "Connection: close" + CRLF +
+                CRLF;
 
         InvalidClient client = new InvalidClient(request);
 
@@ -701,8 +725,12 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
         getTomcatInstance().getConnector().setProperty("rejectIllegalHeader", Boolean.toString(rejectIllegalHeader));
 
         String[] request = new String[1];
-        request[0] = "POST /test HTTP/1.1" + CRLF + "Host: localhost:8080" + CRLF + "Content-Length: 12\u000734" +
-                CRLF + "Connection: close" + CRLF + CRLF;
+        request[0] =
+                "POST /test HTTP/1.1" + CRLF +
+                "Host: localhost:8080" + CRLF +
+                "Content-Length: 12\u000734" + CRLF +
+                "Connection: close" + CRLF +
+                CRLF;
 
         InvalidClient client = new InvalidClient(request);
 
@@ -719,7 +747,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
         private final String[] request;
 
-        InvalidClient(String[] request) {
+        public InvalidClient(String[] request) {
             this.request = request;
         }
 
@@ -754,54 +782,4 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
             return true;
         }
     }
-
-
-    private static final class Client extends SimpleHttpClient {
-
-        Client(int port) {
-            setPort(port);
-        }
-
-        @Override
-        public boolean isResponseBodyOK() {
-            return getResponseBody().contains("test - data");
-        }
-    }
-
-
-    @Test
-    public void testInvalidHeader02() throws Exception {
-        Tomcat tomcat = getTomcatInstance();
-
-        // This setting means the connection will be closed at the end of the
-        // request
-        Assert.assertTrue(tomcat.getConnector().setProperty("maxKeepAliveRequests", "1"));
-
-        // No file system docBase required
-        Context ctx = getProgrammaticRootContext();
-
-        // Add servlet
-        Tomcat.addServlet(ctx, "TesterServlet", new TesterServlet());
-        ctx.addServletMappingDecoded("/foo", "TesterServlet");
-
-        tomcat.start();
-
-        // @formatter:off
-        String request =
-                "GET /foo HTTP/1.1" + SimpleHttpClient.CRLF +
-                "Host: localhost" + SimpleHttpClient.CRLF +
-                ":b" + SimpleHttpClient.CRLF +
-                "X-Dummy:b" + SimpleHttpClient.CRLF +
-                SimpleHttpClient.CRLF;
-        // @formatter:on
-
-        Client client = new Client(tomcat.getConnector().getLocalPort());
-        client.setRequest(new String[] { request });
-
-        client.connect(10000, 600000);
-        client.processRequest();
-
-        // Expected response is a 400 response.
-        Assert.assertTrue(client.getResponseLine(), client.isResponse400());
-    }
 }
